﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;

using ProjectBoat_bae;
using ProjectBoat_bae.Entities;
namespace ProjectBoat_bae.DrawningObjects
{
    public class Drawningboat
    {

        // Класс-сущность
        public EntityBoat? EntityBoat { get; protected set; }

        // Ширина окна
        private int _pictureWidth;

        // Высота окна
        private int _pictureHeight;

        // Левая координата прорисовки
        protected int _startPosX;

        // Верхняя кооридната прорисовки 
        protected int _startPosY;

        // Ширина прорисовки 
        protected readonly int _boatWidth = 80;

        // Высота прорисовки 
        protected readonly int _boatHeight = 50;

        public int GetPosX => _startPosX;
        public int GetPosY => _startPosY;
        public int GetWidth => _boatWidth;
        public int GetHeight => _boatHeight;

        // Конструктор
        public Drawningboat(int speed, double weight, Color bodyColor, int
        width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (width < _pictureWidth  || height < _pictureHeight)
            {
                return;
            }
            EntityBoat = new EntityBoat(speed, weight, bodyColor);
        }

        protected Drawningboat(int speed, double weight, Color bodyColor, int
        width, int height, int boatWidth, int boatHeight)
        {
            if (width < _boatWidth || height < _boatHeight)
            {
                return;
            }

            _pictureWidth = width;
            _pictureHeight = height;
            _boatWidth = boatWidth;
            _boatHeight = boatHeight;
            EntityBoat = new EntityBoat(speed, weight, bodyColor);
        }

        // Установка позиции
        public void SetPosition(int x, int y)
        {
            _startPosX = Math.Min(x, _pictureWidth - _boatWidth);
            _startPosY = Math.Min(y, _pictureHeight - _boatHeight);
        }

        // Прорисовка объекта
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityBoat == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            //корпус
            Brush br = new SolidBrush(EntityBoat.BodyColor);
            g.FillRectangle(br, _startPosX + 20, _startPosY + 5, 70, 50);

            //мотор
            Brush brRed = new SolidBrush(EntityBoat.BodyColor);
            g.FillEllipse(brRed, _startPosX + 7, _startPosY + 12, 35, 35);

            //стекла
            Brush brBlue = new SolidBrush(Color.LightBlue);
            g.FillRectangle(brBlue, _startPosX + 70, _startPosY + 10, 5,
            40);
            g.FillRectangle(brBlue, _startPosX + 35, _startPosY + 8, 35, 2);
            g.FillRectangle(brBlue, _startPosX + 35, _startPosY + 51, 35, 2);
        }

        // Проверка, что объект может переместится по указанному направлению
        public bool CanMove(DiretionType direction)
        {
            if (EntityBoat == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DiretionType.Left => _startPosX - EntityBoat.Step > 0,
                //вверх
                DiretionType.Up => _startPosY - EntityBoat.Step > 0,
                // вправо
                DiretionType.Right => _startPosX + EntityBoat.Step < _pictureWidth,
                //вниз
                DiretionType.Down => _startPosY + EntityBoat.Step < _pictureHeight,
                _ => false
            };
        }        
        
        // Изменение направления перемещения
        public void MoveTransport(DiretionType direction)
        {
            if (!CanMove(direction) || EntityBoat == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DiretionType.Left:
                    if (_startPosX - EntityBoat.Step > 0)
                    {
                        _startPosX -= (int)EntityBoat.Step;
                    }
                    break;
                //вверх
                case DiretionType.Up:
                    if (_startPosY - EntityBoat.Step > 0)
                    {
                        _startPosY -= (int)EntityBoat.Step;
                    }
                    break;
                // вправо
                case DiretionType.Right:
                    if (_startPosX + EntityBoat.Step + _boatWidth < _pictureWidth)
                    {
                        _startPosX += (int)EntityBoat.Step;
                    }
                    break;
                //вниз
                case DiretionType.Down:
                    if (_startPosY + EntityBoat.Step + _boatHeight < _pictureHeight)
                    {
                        _startPosY += (int)EntityBoat.Step;
                    }
                    break;
            }
        }
    }     
}
