﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace ProjectBoat_bae.MovementStrategy
{
    public abstract class AbstractStrategy
    {
     
        // Перемещаемый объект
      
        private IMoveableObject? _moveableObject;
      
        // Статус перемещения
        private Status _state = Status.NotInit;
        // Ширина поля
        protected int FieldWidth { get; private set; }
        // Высота поля
        protected int FieldHeight { get; private set; }
        // Статус перемещения
        public Status GetStatus() { return _state; }

        public void SetData(IMoveableObject moveableObject, int width, int
        height)
        {
            if (moveableObject == null)
            {
                _state = Status.NotInit;
                return;
            }
            _state = Status.InProgress;
            _moveableObject = moveableObject;
            FieldWidth = width;
            FieldHeight = height;
        }
      
        // Шаг перемещения
        public void MakeStep()
        {
            if (_state != Status.InProgress)
            {
                return;
            }
            if (IsTargetDestinaion())
            {
                _state = Status.Finish;
                return;
            }
            MoveToTarget();
        }

    // Перемещение влево    
    protected bool MoveLeft() => MoveTo(DiretionType.Left);

    // Перемещение вправо
    protected bool MoveRight() => MoveTo(DiretionType.Right);

    // Перемещение вверх
    protected bool MoveUp() => MoveTo(DiretionType.Up);

    // Перемещение вниз
    protected bool MoveDown() => MoveTo(DiretionType.Down);
 
        // Параметры объекта
        protected ObjectParameters? GetObjectParameters =>
       _moveableObject?.GetObjectPosition;

        // Шаг объекта
        protected int? GetStep()
        {
            if (_state != Status.InProgress)
            {
                return null;
            }
            return _moveableObject?.GetStep;
        }
        // Перемещение к цели
        protected abstract void MoveToTarget();
        // Достигнута ли цель
        protected abstract bool IsTargetDestinaion();
        // Попытка перемещения в требуемом направлении
        private bool MoveTo(DiretionType directionType)
        {
            if (_state != Status.InProgress)
            {
                return false;
            }
            if (_moveableObject?.CheckCanMove(directionType) ?? false)
            {
                _moveableObject.MoveObject(directionType);
                return true;
            }
            return false;
        }
    }
}
